@@ -176,18 +176,17 @@ def service_redeploy(identifiers, tag):
 
 
 def service_run(image, name, cpu_shares, memory, memory_swap, target_num_containers, run_command, entrypoint,
-                container_ports, container_envvars, linked_to_service, linked_to_container, autorestart, autoreplace,
+                container_ports, container_envvars, linked_to_service,  autorestart, autoreplace,
                 autodestroy, roles, sequential):
     try:
         ports = utils.parse_ports(container_ports)
         envvars = utils.parse_envvars(container_envvars)
         links_service = utils.parse_links(linked_to_service, 'to_service')
-        links_container = utils.parse_links(linked_to_container, 'to_container')
         service = tutum.Service.create(image=image, name=name, cpu_shares=cpu_shares,
                                        memory=memory, memory_swap=memory_swap,
                                        target_num_containers=target_num_containers, run_command=run_command,
                                        entrypoint=entrypoint, container_ports=ports, container_envvars=envvars,
-                                       linked_to_service=links_service, linked_to_container=links_container,
+                                       linked_to_service=links_service,
                                        autorestart=autorestart, autoreplace=autoreplace, autodestroy=autodestroy,
                                        roles=roles, sequential_deployment=sequential)
         service.save()
@@ -499,11 +498,19 @@ def image_push(name, public):
 
         if repository:
             repository = filter(None, repository.split('/'))[-1]
+        tag = None
+        if ':' in repository:
+            tag = repository.split(':')[-1]
+            repository = repository.replace(':%s' % tag, '')
         repository = '%s/%s/%s' % (registry.split('//')[-1].split('/')[0], user, repository)
 
-        try:
+        if tag:
+            print ('Tagging %s as %s:%s ...' % (name, repository, tag))
+        else:
             print('Tagging %s as %s ...' % (name, repository))
-            docker_client.tag(name, repository)
+
+        try:
+            docker_client.tag(name, repository, tag=tag)
         except Exception as e:
             print(e, file=sys.stderr)
             sys.exit(EXCEPTION_EXIT_CODE)
@@ -511,7 +518,7 @@ def image_push(name, public):
         stream = docker_client.push(repository, stream=True)
         try:
             utils.stream_output(stream, sys.stdout)
-        except docker.client.APIError as e:
+        except docker.errors.APIError as e:
             print(e.explanation, file=sys.stderr)
             sys.exit(EXCEPTION_EXIT_CODE)
         except Exception as e:
